﻿using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using WpfApp1_drives.BL;

namespace WpfApp1_drives.VM
{
    class MainWindowViewModel : ViewModelBase
    {
        private IDirectoryLogic directoryLogic;

        public IEnumerable<DirectoryEntry> Entries { get; private set; }
        public string CurrentDirectory { get; private set; }
        public ICommand SelectEntryCommand { get; private set; }

        private DirectoryEntry selectedEntry;
        public DirectoryEntry SelectedEntry
        {
            get => selectedEntry; set => Set(ref selectedEntry, value);
        }


        public MainWindowViewModel(IDirectoryLogic logic, string currentDirectory)
        { 
            this.directoryLogic = logic;
            CurrentDirectory = currentDirectory;

            Entries = directoryLogic.CollectDirectoryEntries(currentDirectory);

            SelectEntryCommand = new RelayCommand(() => directoryLogic.SelectEntry(SelectedEntry), true);

            // RelayCommand uses weak delegate references 
            //      => it does not force the GC to keep the lambda in memory
            //      => IF the lambda uses datafield references, then GC will not free up the lambda expression
            //      => IF the lambda uses only method parameters, then GC will free up the lambda expression 
            //          Must use TRUE as second parameter in RelayCommand ctor !!!
            //          This might cause memory leak !!!
            //          See lecture/weak delegates for details
            //          (observer design pattern + delegates)

            // Hard delegate references are possible since MvvmLight 5.4.0
            //      with the TRUE second parameter in the RelayCommand ctor
            // http://galasoft.ch/s/mvvmweakaction
            // "TRUE is needed because any GC cycle can recycle local lambda function if the lambda uses only local parameters"
            /*
            SelectEntryCommand = new RelayCommand(
                () => {
                    // directoryLogic.SelectEntry(SelectedEntry); 
                    // references the property/datafield => TRUE as second parameter is not needed

                    // logic.SelectEntry(SelectedEntry); 
                    // references the paramter => TRUE as second parameter is YES, needed!!!
                });
             */
        }

        public MainWindowViewModel(IEntrySelectorService service, string currentDirectory)
            : this(new DirectoryLogic(service), currentDirectory)
        // TODO: extract the DirectoryLogic ctor call into a layer-independent factory, IoC OR singleton???
        {
        }
    }
}
